import unittest

from fake import FILE_REGISTRY

from address.factories import AddressFactory, PersonFactory
from address.models import Address, Person
from data import CITIES, REGIONS

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2023-2024 Artur Barseghyan"
__license__ = "MIT"
__all__ = ("AddressFakerTestCase",)


class AddressFakerTestCase(unittest.TestCase):
    def tearDown(self):
        FILE_REGISTRY.clean_up()

    def test_sub_factory(self) -> None:
        person = PersonFactory()

        # Testing SubFactory
        self.assertIsInstance(person.address, Address)
        self.assertIsInstance(person.address.id, int)

        # Testing Factory
        self.assertIsInstance(person.id, int)

        # Testing hooks
        self.assertTrue(
            hasattr(person, "_pre_save_called") and person._pre_save_called
        )
        self.assertTrue(
            hasattr(person, "_post_save_called") and person._post_save_called
        )
        self.assertTrue(
            hasattr(person.address, "_pre_save_called")
            and person.address._pre_save_called
        )
        self.assertTrue(
            hasattr(person.address, "_post_save_called")
            and person.address._post_save_called
        )

        # Testing batch creation
        persons = PersonFactory.create_batch(5)
        self.assertEqual(len(persons), 5)
        self.assertIsInstance(persons[0], Person)

        # Testing custom addresses
        address = AddressFactory()
        self.assertIn(address.city, CITIES)
        self.assertIn(address.region, REGIONS)



from sqlalchemy.orm import scoped_session, sessionmaker
from sqlmodel import SQLModel, create_engine

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2023-2024 Artur Barseghyan"
__license__ = "MIT"
__all__ = (
    "DATABASE_URL",
    "ENGINE",
    "SESSION",
)

# SQLAlchemy
DATABASE_URL = "sqlite:///test_database.db"
ENGINE = create_engine(DATABASE_URL)
SESSION = scoped_session(sessionmaker(bind=ENGINE))


def create_tables():
    """Create tables."""
    SQLModel.metadata.create_all(ENGINE)


def get_db():
    """Get database connection."""
    session = SESSION()
    try:
        yield session
        session.commit()
    finally:
        session.close()


if __name__ == "__main__":
    create_tables()



# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html
import os
import sys

sys.path.insert(0, os.path.abspath(".."))


# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information

try:
    import fake

    version = fake.__version__
    project = fake.__title__
    copyright = fake.__copyright__
    author = fake.__author__
except ImportError:
    version = "0.1"
    project = "fake.py"
    copyright = "2023, Artur Barseghyan <artur.barseghyan@gmail.com>"
    author = "Artur Barseghyan <artur.barseghyan@gmail.com>"

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.viewcode",
    "sphinx.ext.todo",
    "sphinx_no_pragma",
]

templates_path = ["_templates"]
exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]

language = "en"

release = version

# The suffix of source filenames.
source_suffix = {
    ".rst": "restructuredtext",
}

pygments_style = "sphinx"

# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

html_theme = "sphinx_rtd_theme"
html_static_path = ["_static"]
# html_extra_path = ["examples"]

prismjs_base = "//cdnjs.cloudflare.com/ajax/libs/prism/1.29.0"

html_css_files = [
    f"{prismjs_base}/themes/prism.min.css",
    f"{prismjs_base}/plugins/toolbar/prism-toolbar.min.css",
    "https://cdn.jsdelivr.net/gh/barseghyanartur/jsphinx@1.3.4/src/css/sphinx_rtd_theme.css",
]

html_js_files = [
    f"{prismjs_base}/prism.min.js",
    f"{prismjs_base}/plugins/autoloader/prism-autoloader.min.js",
    f"{prismjs_base}/plugins/toolbar/prism-toolbar.min.js",
    f"{prismjs_base}/plugins/copy-to-clipboard/prism-copy-to-clipboard.min.js",
    "https://cdn.jsdelivr.net/gh/barseghyanartur/jsphinx@1.3.4/src/js/download_adapter.js",
]

# -- Options for todo extension ----------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/extensions/todo.html#configuration

todo_include_todos = True

# -- Options for Epub output  ----------------------------------------------
epub_title = project
epub_author = author
epub_publisher = "GitHub"
epub_copyright = copyright
epub_identifier = "https://github.com/barseghyanartur/fake.py"  # URL or ISBN
epub_scheme = "URL"  # or "ISBN"
epub_uid = "https://github.com/barseghyanartur/fake.py"



from fake import ModelFactory, SubFactory, post_save, pre_save
from override_default_data import FACTORY as OVERRIDE_DEFAULT_DATA_FACTORY

from address.models import Address, Person
from fake_address import FACTORY as ADDRESS_FACTORY

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2023-2024 Artur Barseghyan"
__license__ = "MIT"
__all__ = (
    "AddressFactory",
    "PersonFactory",
)


class AddressFactory(ModelFactory):
    id = ADDRESS_FACTORY.pyint()
    address_line = ADDRESS_FACTORY.address_line()
    postal_code = ADDRESS_FACTORY.postal_code()
    city = ADDRESS_FACTORY.city()
    region = ADDRESS_FACTORY.region()

    class Meta:
        model = Address

    @pre_save
    def _pre_save_method(self, instance):
        instance._pre_save_called = True

    @post_save
    def _post_save_method(self, instance):
        instance._post_save_called = True


class PersonFactory(ModelFactory):
    id = OVERRIDE_DEFAULT_DATA_FACTORY.pyint()
    first_name = OVERRIDE_DEFAULT_DATA_FACTORY.first_name()
    last_name = OVERRIDE_DEFAULT_DATA_FACTORY.last_name()
    email = OVERRIDE_DEFAULT_DATA_FACTORY.email()
    dob = OVERRIDE_DEFAULT_DATA_FACTORY.date()
    address = SubFactory(AddressFactory)

    class Meta:
        model = Person

    @pre_save
    def _pre_save_method(self, instance):
        instance._pre_save_called = True

    @post_save
    def _post_save_method(self, instance):
        instance._post_save_called = True



from datetime import datetime

from fake import FILE_REGISTRY, run_async_in_thread, xor_transform
from tortoise.contrib import test
from tortoise.contrib.test import finalizer, initializer

from article.factories import ArticleFactory, UserFactory
from article.models import Article, User

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2023-2024 Artur Barseghyan"
__license__ = "MIT"
__all__ = ("FactoriesTestCase",)


class FactoriesTestCase(test.TestCase):
    @classmethod
    def setUpClass(cls):
        initializer(["article.models"], db_url="sqlite://:memory:")
        # Your setup code...

    @classmethod
    def tearDownClass(cls):
        finalizer()
        # Your teardown code...

    def tearDown(self):
        FILE_REGISTRY.clean_up()

    def test_sub_factory(self) -> None:
        with self.subTest("SubFactory"):
            article = ArticleFactory()

            # Testing SubFactory
            self.assertIsInstance(article.author, User)
            self.assertIsInstance(article.author.id, int)
            self.assertIsInstance(article.author.is_staff, bool)
            self.assertIsInstance(article.author.date_joined, datetime)

            # Testing Factory
            self.assertIsInstance(article.id, int)
            self.assertIsInstance(article.slug, str)

            # Testing hooks
            self.assertTrue(
                hasattr(article, "_pre_save_called")
                and article._pre_save_called
            )
            self.assertTrue(
                hasattr(article, "_post_save_called")
                and article._post_save_called
            )
            self.assertTrue(
                hasattr(article.author, "_pre_save_called")
                and article.author._pre_save_called
            )
            self.assertTrue(
                hasattr(article.author, "_post_save_called")
                and article.author._post_save_called
            )

            # Testing batch creation
            articles = ArticleFactory.create_batch(5)
            self.assertEqual(len(articles), 5)
            self.assertIsInstance(articles[0], Article)

        with self.subTest("SubFactory nested params"):
            article = ArticleFactory(author__username="admin")

            # Testing SubFactory
            self.assertIsInstance(article.author, User)
            self.assertIsInstance(article.author.id, int)
            self.assertIsInstance(article.author.is_staff, bool)
            self.assertIsInstance(article.author.date_joined, datetime)
            self.assertEqual(article.author.username, "admin")

            # Testing Factory
            self.assertIsInstance(article.id, int)
            self.assertIsInstance(article.slug, str)

            # Testing hooks
            self.assertTrue(
                hasattr(article, "_pre_save_called")
                and article._pre_save_called
            )
            self.assertTrue(
                hasattr(article, "_post_save_called")
                and article._post_save_called
            )
            self.assertTrue(
                hasattr(article.author, "_pre_save_called")
                and article.author._pre_save_called
            )
            self.assertTrue(
                hasattr(article.author, "_post_save_called")
                and article.author._post_save_called
            )

            # Testing batch creation
            articles = ArticleFactory.create_batch(5)
            self.assertEqual(len(articles), 5)
            self.assertIsInstance(articles[0], Article)

    def test_pre_save_and_post_save(self) -> None:
        """Test PreSave and PostSave."""
        user = UserFactory(is_staff=True, is_active=True)
        self.assertEqual(
            xor_transform(user.password),
            "test1234",
        )

        async def get_first_group_name():
            first_group = await user.groups.all().first()
            if first_group:
                return first_group.name
            return None

        group_name = run_async_in_thread(get_first_group_name())
        self.assertEqual(group_name, "TestGroup1234")



"""
Configuration hooks for `pytest`. Normally this wouldn't be necessary,
but since `pytest-rst` is used, we want to clean-up files generated by
running documentation tests. Therefore, this hook, which simply
calls the `clean_up` method of the `FILE_REGISTRY` instance.
"""

import pytest
from fake import FILE_REGISTRY

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2023-2024 Artur Barseghyan"
__license__ = "MIT"
__all__ = (
    "pytest_runtest_setup",
    "pytest_runtest_teardown",
)


def pytest_collection_modifyitems(session, config, items):
    """Modify test items during collection."""
    for item in items:
        try:
            from pytest_rst import RSTTestItem

            if isinstance(item, RSTTestItem):
                # Dynamically add marker to RSTTestItem tests
                item.add_marker(pytest.mark.django_db(transaction=True))
        except ImportError:
            pass


def pytest_runtest_setup(item):
    """Setup before test runs."""
    try:
        from pytest_rst import RSTTestItem

        if isinstance(item, RSTTestItem):
            pass
    except ImportError:
        pass


def pytest_runtest_teardown(item, nextitem):
    """Clean up after test ends."""
    try:
        from pytest_rst import RSTTestItem

        if isinstance(item, RSTTestItem):
            FILE_REGISTRY.clean_up()
    except ImportError:
        pass



from pathlib import Path
from typing import Any, Dict

from fake import (
    FACTORY,
    FAKER,
    FileSystemStorage,
    PostSave,
    PreInit,
    PreSave,
    PydanticModelFactory,
    SubFactory,
    post_save,
    pre_init,
    pre_save,
    slugify,
    trait,
)

from article.models import Article, Group, User

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2023-2024 Artur Barseghyan"
__license__ = "MIT"
__all__ = (
    "ArticleFactory",
    "GroupFactory",
    "UserFactory",
)

# Build paths inside the project like this: BASE_DIR / 'subdir'.
BASE_DIR = Path(__file__).resolve().parent.parent
MEDIA_ROOT = BASE_DIR / "media"

STORAGE = FileSystemStorage(root_path=MEDIA_ROOT, rel_path="tmp")

CATEGORIES = (
    "art",
    "technology",
    "literature",
)
TAGS = (
    "painting",
    "photography",
    "ai",
    "data-engineering",
    "fiction",
    "poetry",
    "manual",
)


class GroupFactory(PydanticModelFactory):
    id = FACTORY.pyint()
    name = FACTORY.word()

    class Meta:
        model = Group
        get_or_create = ("name",)


def set_password(user: User, password: str) -> None:
    user.set_password(password)


def add_to_group(user: User, name: str) -> None:
    group = GroupFactory(name=name)
    user.groups.add(group)


def set_username(data: Dict[str, Any]) -> None:
    first_name = slugify(data["first_name"])
    last_name = slugify(data["last_name"])
    data["username"] = f"{first_name}_{last_name}_{FAKER.pystr().lower()}"


class UserFactory(PydanticModelFactory):
    id = FACTORY.pyint()
    username = PreInit(set_username)
    first_name = FACTORY.first_name()
    last_name = FACTORY.last_name()
    email = FACTORY.email()
    date_joined = FACTORY.date_time()
    last_login = FACTORY.date_time()
    is_superuser = False
    is_staff = False
    is_active = FACTORY.pybool()
    password = PreSave(set_password, password="test1234")  # type: ignore
    group = PostSave(add_to_group, name="TestGroup1234")  # type: ignore

    class Meta:
        model = User

    @trait
    def is_admin_user(self, instance: User) -> None:
        instance.is_superuser = True
        instance.is_staff = True
        instance.is_active = True

    @pre_init
    def _pre_init_method(self, data):
        # For testing purposes only
        data["_pre_init_called"] = True

    @pre_save
    def _pre_save_method(self, instance):
        # For testing purposes only
        instance._pre_save_called = True

    @post_save
    def _post_save_method(self, instance):
        # For testing purposes only
        instance._post_save_called = True


def set_headline(data) -> None:
    data["headline"] = data["content"][:25]


class ArticleFactory(PydanticModelFactory):
    id = FACTORY.pyint()
    title = FACTORY.sentence()
    slug = FACTORY.slug()
    content = FACTORY.text()
    headline = PreInit(set_headline)
    category = FACTORY.random_choice(elements=CATEGORIES)
    pages = FACTORY.pyint(min_value=1, max_value=100)  # type: ignore
    image = FACTORY.png_file(storage=STORAGE)
    pub_date = FACTORY.date()
    safe_for_work = FACTORY.pybool()
    minutes_to_read = FACTORY.pyint(min_value=1, max_value=10)
    author = SubFactory(UserFactory)
    tags = FACTORY.random_sample(elements=TAGS, length=3)

    class Meta:
        model = Article

    @pre_init
    def set_auto_minutes_to_read(self, data: Dict[str, Any]) -> None:
        data["auto_minutes_to_read"] = data["pages"]

    @pre_init
    def _pre_init_method(self, data):
        # For testing purposes only
        data["_pre_init_called"] = True

    @pre_save
    def _pre_save_method(self, instance):
        # For testing purposes only
        instance._pre_save_called = True

    @post_save
    def _post_save_method(self, instance):
        # For testing purposes only
        instance._post_save_called = True



from band.models import Band
from fake import ModelFactory

from fake_band import FACTORY

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2023-2024 Artur Barseghyan"
__license__ = "MIT"
__all__ = ("BandFactory",)


class BandFactory(ModelFactory):
    id = FACTORY.pyint()
    name = FACTORY.band_name()

    class Meta:
        model = Band
        get_or_create = ("name",)



from pathlib import Path

from fake import FAKER, FileSystemStorage, TextPdfGenerator

BASE_DIR = Path(__file__).resolve().parent.parent
MEDIA_ROOT = BASE_DIR / "media"

STORAGE = FileSystemStorage(root_path=MEDIA_ROOT, rel_path="tmp")

pdf_file = FAKER.pdf_file(generator=TextPdfGenerator, storage=STORAGE)



from datetime import date, datetime
from typing import List, Optional, Set

from fake import xor_transform
from pydantic import BaseModel, Field

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2023-2024 Artur Barseghyan"
__license__ = "MIT"
__all__ = (
    "Article",
    "Group",
    "User",
)


class Group(BaseModel):
    id: int
    name: str

    class Config:
        allow_mutation = False

    def __hash__(self):
        return hash((self.id, self.name))

    def __eq__(self, other):
        if isinstance(other, Group):
            return self.id == other.id and self.name == other.name
        return False


class User(BaseModel):
    id: int
    username: str = Field(max_length=255)
    first_name: str = Field(max_length=255)
    last_name: str = Field(max_length=255)
    email: str = Field(max_length=255)
    date_joined: datetime = Field(default_factory=datetime.utcnow)
    last_login: Optional[datetime] = None
    password: Optional[str] = Field("", max_length=255)
    is_superuser: bool = Field(default=False)
    is_staff: bool = Field(default=False)
    is_active: bool = Field(default=True)
    groups: Set[Group] = Field(default_factory=set)

    class Config:
        extra = "allow"  # For testing purposes only

    def __str__(self):
        return self.username

    def set_password(self, password: str) -> None:
        self.password = xor_transform(password)


class Article(BaseModel):
    id: int
    title: str = Field(max_length=255)
    slug: str = Field(max_length=255, unique=True)
    content: str
    headline: str
    category: str = Field(max_length=255)
    pages: int
    auto_minutes_to_read: int
    author: User
    image: Optional[str] = None  # Use str to represent the image path or URL
    pub_date: date = Field(default_factory=date.today)
    safe_for_work: bool = False
    minutes_to_read: int = 5
    tags: List[str] = Field(default_factory=list)

    class Config:
        extra = "allow"  # For testing purposes only

    def __str__(self):
        return self.title



from sqlalchemy import create_engine
from sqlalchemy.orm import scoped_session, sessionmaker

__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2023-2024 Artur Barseghyan"
__license__ = "MIT"
__all__ = (
    "DATABASE_URL",
    "ENGINE",
    "SESSION",
)

# SQLAlchemy
DATABASE_URL = "sqlite:///test_database.db"
ENGINE = create_engine(DATABASE_URL)
SESSION = scoped_session(sessionmaker(bind=ENGINE))
